/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details. */

//TODO: Move away from SFML, it's not up to the task.

#include "Game.hpp"
#include <cmath>
#include <utility>
#include <GL/glu.h>
#include <GL/gl.h>
#include <iostream>

Game::Game(Player& p, unsigned width, unsigned height, unsigned points) : triangulations(20), player(p), window(sf::VideoMode(width, height, 32), "Fuzzy game"), running(true), points(std::max(4u,points)), width(width), height(height), generator(1.0), captureMouse(false)
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glClearDepth(1.f);
	glClearColor(0.f, 0.f, 0.f, 0.f);
	gluOrtho2D(-1.f, 1.f, 1.f, -1.f);
	
	//Generate triangulations
	for(std::vector<Triangulation>::iterator i = triangulations.begin(); i != triangulations.end(); ++i) {
		i->clear();
		i->insert(Point(-1,-1));
		i->insert(Point(-1, 1));
		i->insert(Point( 1,-1));
		i->insert(Point( 1, 1));
		CGAL::copy_n(generator, points-4, std::back_inserter(*i));
	}
	
}//Fill out

void Game::processEvents()
{
	sf::Event event;
	while (window.GetEvent(event)) {
		if (event.Type == sf::Event::Closed)
			running = false;
		else if (event.Type == sf::Event::KeyPressed) {
			switch(event.Key.Code) {
				case (sf::Key::Escape):
					running=false;
					break;
				case (sf::Key::Left):
					player.setLeft(true);
					break;
				case (sf::Key::Right):
					player.setRight(true);
					break;
				case (sf::Key::Up):
					player.setForward(true);
					break;
				case (sf::Key::Down):
					player.setBackward(true);
					break;
			}
		} else if (event.Type == sf::Event::KeyReleased) {
			switch(event.Key.Code) {
				case (sf::Key::Left):
					player.setLeft(false);
					break;
				case (sf::Key::Right):
					player.setRight(false);
					break;
				case (sf::Key::Up):
					player.setForward(false);
					break;
				case (sf::Key::Down):
					player.setBackward(false);
					break;
			}
		} else if (event.Type == sf::Event::MouseButtonPressed && event.MouseButton.Button == sf::Mouse::Left) {
			if (captureMouse) {
				captureMouse = false;
				window.ShowMouseCursor(true);
			} else {
				captureMouse = true;
				window.ShowMouseCursor(false);
				window.SetCursorPosition(window.GetWidth()/2, window.GetHeight()/2);
			}
		} else if (event.Type == sf::Event::MouseMoved && captureMouse) {
			player.turn(static_cast<int>(event.MouseMove.X)-static_cast<int>(window.GetWidth())/2, static_cast<int>(window.GetHeight())/2-static_cast<int>(event.MouseMove.Y));
			window.SetCursorPosition(window.GetWidth()/2, window.GetHeight()/2);
		}/* else if (event.Type == sf::Event::Resized) {
			width = window.GetWidth();
			height = window.GetHeight();
		}*///Drawing relies on the old coords, mouse movement needs the new ones. Drawing needs them more often.
	}
}

void Game::play()
{
	//menu?
	window.SetFramerateLimit(60);
	//window.UseVerticalSync(true);
	double minDist = 0.0, maxDist=8.0;
	while(running) {
		for(std::vector<Triangulation>::iterator vi=triangulations.begin(); vi != triangulations.end() && running; ++vi) {

			Triangulation& triangulation = *vi;
			processEvents();
			player.update();
			std::pair<double,double> bounds = player.look(triangulation.tds().vertices_begin(), triangulation.tds().vertices_end());
			
			//double range = (bounds.second-bounds.first);
			minDist = (3*minDist + bounds.first)/4;
			maxDist = (3*maxDist+ bounds.second)/4;
			double range = maxDist-minDist;
			//Far - Near, NOTE: Distance currently bounded to stop recursion getting too deep, not really necessary.
			
			glBegin(GL_TRIANGLES);
			for(Triangulation::Finite_faces_iterator j = triangulation.finite_faces_begin(); j != triangulation.finite_faces_end(); ++j) {
				for(unsigned vertex = 0; vertex < 3; ++vertex) {
					double level = std::max(0.0,std::min(1.0,((j->vertex(vertex)->info()-bounds.first)/range)));
					glColor3d(level, level, sqrt(level));
					//NOTE: Square/sqrt level in a colour to tinge the scene without losing black&white near/far.
					glVertex2d(j->vertex(vertex)->point().x(), j->vertex(vertex)->point().y());
				}
			}
			glEnd();
			window.Display();
		}
	}
}
